@@ -1057,6 +1057,23 @@ pre_object(Obj, _, _, _) :-
 :- rdf_meta
     pre_ground_object(+, o).
 
+pre_ground_object(Val@Lang,  literal(lang(Lang0, Val0))) :-
+    !,
+    downcase_atom(Lang, Lang0),
+    in_lang_string(Val, Val0).
+pre_ground_object(Val^^Type, literal(type(Type0, Val0))) :-
+    !,
+    in_type(Type, Val, Type0, Val0).
+pre_ground_object(literal(Lit0), literal(Lit)) :-
+    old_literal(Lit0, Lit),
+    !.
+% Interpret `false' and `true' as the Boolean values.
+pre_ground_object(false, literal(type(xsd:boolean, false))) :- !.
+pre_ground_object(true, literal(type(xsd:boolean, true))) :- !.
+pre_ground_object(Atom, URI) :-
+    atom(Atom),
+    !,
+    URI = Atom.
 % Interpret Prolog integer as xsd:integer.
 pre_ground_object(Int, Object) :-
     integer(Int),
@@ -1076,9 +1093,6 @@ pre_ground_object(String, Object) :-
     !,
     rdf_equal(Object, literal(type(xsd:string, Atom))),
     atom_string(Atom, String).
-% Interpret `false' and `true' as the Boolean values.
-pre_ground_object(false, literal(type(xsd:boolean, false))) :- !.
-pre_ground_object(true, literal(type(xsd:boolean, true))) :- !.
 % Interpret date(Y,M,D) as xsd:date,
 %           date_time(Y,M,D,HH,MM,SS) as xsd:dateTime,
 %           date_time(Y,M,D,HH,MM,SS,TZ) as xsd:dateTime,
@@ -1090,20 +1104,6 @@ pre_ground_object(Term, literal(type(Type, Atom))) :-
     !,
     xsd_time_string(Term, Type, String),
     atom_string(Atom, String).
-pre_ground_object(Val@Lang,  literal(lang(Lang0, Val0))) :-
-    !,
-    downcase_atom(Lang, Lang0),
-    in_lang_string(Val, Val0).
-pre_ground_object(Val^^Type, literal(type(Type0, Val0))) :-
-    !,
-    in_type(Type, Val, Type0, Val0).
-pre_ground_object(Atom, URI) :-
-    atom(Atom),
-    !,
-    URI = Atom.
-pre_ground_object(literal(Lit0), literal(Lit)) :-
-    old_literal(Lit0, Lit),
-    !.
 pre_ground_object(Value, _) :-
     type_error(rdf_object, Value).
 
@@ -1259,6 +1259,17 @@ in_number(double, _Domain, _, Val, Val0) :-
     !,
     xsd_number_string(ValF, ValS),
     atom_string(Val0, ValS).
+in_number(decimal, _Domain, _, Val, Val0) :-
+    number(Val),
+    !,
+    ValF is float(Val),
+    atom_number(Val0, ValF).
+in_number(decimal, _Domain, _, Val, Val0) :-
+    atomic(Val),
+    xsd_number_string(Num, Val),
+    ValF is float(Num),
+    !,
+    atom_number(Val0, ValF).
 in_number(PrologType, _, _, Val, _) :-
     type_error(PrologType, Val).
 
@@ -1286,7 +1297,7 @@ error:has_type(nonpos, T):-
 
 xsd_numerical(xsd:byte,               between(-128,127),               integer).
 xsd_numerical(xsd:double,             float,                           double).
-xsd_numerical(xsd:decimal,            float,                           double).
+xsd_numerical(xsd:decimal,            float,                           decimal).
 xsd_numerical(xsd:float,              float,                           double).
 xsd_numerical(xsd:int,                between(-2147483648,2147483647), integer).
 xsd_numerical(xsd:integer,            integer,                         integer).
@@ -1543,6 +1554,10 @@ gen_term(O) :-                          % performs double conversion!
 rdf_literal(Term) :-
     ground(Term),
     !,
+    (   boolean(Term)
+    ;   \+ atom(Term)
+    ),
+    !,
     pre_ground_object(Term, Object),
     (rdf_db:rdf(_,_,Object)->true).
 rdf_literal(Term) :-
