@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1997-2022, University of Amsterdam
+    Copyright (c)  1997-2023, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -40,6 +40,7 @@
 #include "pl-allocpool.h"
 #include "pl-mutex.h"
 #include "pl-thread.h"
+#include "pl-prof.h"
 #include "pl-gmp.h"
 
 #ifndef GLOBAL			/* global variables */
@@ -154,11 +155,11 @@ struct PL_global_data
     { int	created;		/* # created hash tables */
       int	destroyed;		/* # destroyed hash tables */
     } indexes;
-#ifdef O_PLMT
-    int		threads_created;	/* # threads created */
-    int		threads_finished;	/* # finished threads */
-    int		engines_created;	/* # engines created */
-    int		engines_finished;	/* # engines threads */
+#ifdef O_ENGINES
+    uint64_t	engines_created;	/* # engines created */
+    uint64_t	engines_finished;	/* # engines threads */
+    uint64_t	threads_created;	/* # threads created */
+    uint64_t	threads_finished;	/* # finished threads */
     double	thread_cputime;		/* Total CPU time of threads */
 #endif
     int		errors;			/* Printed error messages */
@@ -245,7 +246,7 @@ struct PL_global_data
     int		  _loaded;		/* system extensions are loaded */
   } foreign;
 
-#ifdef O_GMP
+#ifdef O_BIGNUM
   struct
   { int initialised;			/* is GMP initialised? */
     int keep_alloc_functions;		/* do not change allocation */
@@ -281,7 +282,7 @@ struct PL_global_data
       struct event_list *onerase;	/* erased clause or dbref */
       struct event_list *onbreak;	/* breakpoint change */
       struct event_list *onframefinish; /* Debugged frame finished */
-#ifdef O_PLMT
+#ifdef O_ENGINES
       struct event_list *onthreadstart;	/* thread start hook */
       struct event_list *onthreadexit;	/* thread exit hook */
 #endif
@@ -318,7 +319,7 @@ struct PL_global_data
     Procedure	strict_equal2;		/* ==/2 */
     Procedure	not_strict_equal2;	/* \==/2 */
     Procedure   arg3;			/* arg/3 */
-    Procedure	exception_hook4;
+    Procedure	exception_hook5;
     Procedure	print_message2;
     Procedure	foreign_registered2;	/* $foreign_registered/2 */
     Procedure	prolog_trace_interception4;
@@ -327,7 +328,6 @@ struct PL_global_data
     Procedure   dcall1;			/* $call/1 */
     Procedure   call1;			/* call/1 */
     Procedure   call3;			/* call/3 */
-    Procedure	setup_call_catcher_cleanup4; /* setup_call_catcher_cleanup/4 */
     Procedure	undefinterc4;		/* $undefined_procedure/4 */
     Procedure   dthread_init0;		/* $thread_init/0 */
     Procedure   dc_call_prolog0;	/* $c_call_prolog/0 */
@@ -350,8 +350,10 @@ struct PL_global_data
     Procedure	answer_count_restraint0;/* $tabling:answer_count_restraint/0 */
     Procedure	radial_restraint0;	/* $tabling:radial_restraint/0 */
     Procedure	tripwire3;		/* $tabling:tripwire/3 */
-#if O_PLMT
+#if O_ENGINES
     Procedure	signal_is_blocked1;	/* $syspreds:signal_is_blocked1/1 */
+#endif
+#if O_PLMT
     Procedure	dgc0;			/* system:$gc/0 */
 #endif
     Procedure	drun_undo1;		/* $syspreds:$run_undo/1 */
@@ -398,27 +400,29 @@ struct PL_global_data
 #ifdef HAVE_TGETENT
   struct
   { int    initialised;			/* initialisation status */
-    char  *_string_area;		/* static area for tgetent */
-    char  *_buf_area;			/* another one */
+    char  *_string_area;		/* static area for tgetstr */
+    char  *_string_area_end;		/* end of _string_area */
+    char  *_buf_area;			/* static area for tgetent */
     Table  _capabilities;		/* User-level capability table */
   } terminal;
 #endif
 
-#ifdef O_PLMT
+#ifdef O_ENGINES
   struct
-  { int			enabled;	/* threads are enabled */
-    Table		mutexTable;	/* Name --> mutex table */
-    int			mutex_next_id;	/* next id for anonymous mutexes */
-#ifdef __WINDOWS__
-    HINSTANCE		instance;	/* Win32 process instance */
-#endif
-    counting_mutex     *mutexes;	/* Registered mutexes */
-    PL_thread_info_t   *free;		/* Free threads */
+  { PL_thread_info_t   *free;		/* Free threads */
     int			highest_allocated; /* Highest with info struct */
     int			thread_max;	/* Size of threads array */
     int			highest_id;	/* Highest Id of life thread  */
     int			peak_id;	/* Highest Id of any thread  */
     PL_thread_info_t  **threads;	/* Pointers to thread-info */
+#ifdef __WINDOWS__
+    HINSTANCE		instance;	/* Win32 process instance */
+#endif
+#ifdef O_PLMT
+    int			enabled;	/* threads are enabled */
+    int			mutex_next_id;	/* next id for anonymous mutexes */
+    Table		mutexTable;	/* Name --> mutex table */
+    counting_mutex     *mutexes;	/* Registered mutexes */
     struct
     { pthread_mutex_t	mutex;
       pthread_cond_t	cond;
@@ -430,8 +434,9 @@ struct PL_global_data
       pthread_cond_t	cond;
     } index;
     linger_list	       *lingering;
+#endif
   } thread;
-#endif /*O_PLMT*/
+#endif /*O_ENGINES*/
 
   struct
   { functor_t dict_functors[CACHED_DICT_FUNCTORS];
@@ -477,9 +482,6 @@ struct PL_local_data
   int		break_level;		/* current break level */
   Stack		outofstack;		/* thread is out of stack */
   int		trim_stack_requested;	/* perform a trim-stack */
-#ifdef O_PLMT
-  int		exit_requested;		/* Thread is asked to exit */
-#endif
   int		in_arithmetic;		/* doing arithmetic */
   int		in_print_message;	/* Inside printMessage() */
   void *	glob_info;		/* pl-glob.c */
@@ -512,7 +514,8 @@ struct PL_local_data
   struct
   { char       *getstr_buffer;		/* getString() buffer */
     size_t	getstr_buffer_size;	/* size of getstr_buffer */
-    struct wic_state *current_state;	/* qlf-creation state */
+    struct wic_state *write_state;	/* qlf-write state */
+    struct wic_state *read_state;	/* qlf-read state */
   } qlf;
 
   struct
@@ -569,9 +572,9 @@ struct PL_local_data
 
   struct
   { uint64_t	inferences;		/* inferences in this thread */
-    uintptr_t	last_cputime;		/* milliseconds last CPU time */
-    uintptr_t	last_systime;		/* milliseconds last SYSTEM time */
-    uintptr_t	last_real_time;		/* Last Real Time (seconds since Epoch) */
+    uint64_t	last_cputime;		/* milliseconds last CPU time */
+    uint64_t	last_systime;		/* milliseconds last SYSTEM time */
+    uint64_t	last_real_time;		/* Last Real Time (seconds since Epoch) */
     double	start_time;		/* When Thread was started */
     double	last_walltime;		/* Last Wall time (m-secs since start) */
     double	user_cputime;		/* User saved CPU time */
@@ -580,7 +583,7 @@ struct PL_local_data
     int		warnings;		/* Printed warning messages */
   } statistics;
 
-#ifdef O_GMP
+#ifdef O_BIGNUM
   struct
   { int		persistent;		/* do persistent operations */
     size_t	allocated;		/* memory allocated */
@@ -592,15 +595,17 @@ struct PL_local_data
 
 #ifdef O_PROFILE
   struct
-  { int		active;			/* profiler is on */
-    int		accounting;		/* we are accounting */
+  { int		accounting;		/* we are accounting */
     int		sum_ok;			/* siblings are counted */
+    prof_status	active;			/* profiler is on */
+    prof_control ports_control;		/* which port counts are generated? */
+    unsigned int sample_period;		/* profile sample period (usecs.) */
     struct call_node *current;		/* `current' node */
     struct call_node *roots;		/* list of root-nodes */
-    uintptr_t	samples;		/* profile samples */
-    uintptr_t	ticks;			/* profile ticks total */
-    uintptr_t	accounting_ticks;	/* Ticks in profCall() and friends */
-    uintptr_t	nodes;			/* #Recorded nodes */
+    uint64_t	samples;		/* profile samples */
+    uint64_t	ticks;			/* profile ticks total */
+    uint64_t	accounting_ticks;	/* Ticks in profCall() and friends */
+    size_t	nodes;			/* #Recorded nodes */
     double	time_at_last_tick;	/* Time at last statistics tick */
     double	time_at_start;		/* Time at last start */
     double	time;			/* recorded CPU time */
@@ -615,6 +620,7 @@ struct PL_local_data
   struct
   { intptr_t	generator;		/* See PL_atom_generator() */
     atom_t	unregistering;		/* See PL_unregister_atom() */
+    int		gc_active;		/* Thread is running atom-gc */
   } atoms;
 
   struct
@@ -627,7 +633,7 @@ struct PL_local_data
   { Buffer	buffered;		/* Buffered events */
     int		delay_nesting;		/* How deeply is delay nested? */
 
-#ifdef O_PLMT
+#ifdef O_ENGINES
     struct
     { struct event_list *onthreadexit;	/* thread exit hook */
     } hook;
@@ -640,7 +646,7 @@ struct PL_local_data
       Number	top;
       Number	max;
     } stack;
-#ifdef O_GMP
+#ifdef O_BIGNUM
     struct
     { gmp_randstate_t state;
       int initialised;
@@ -758,18 +764,26 @@ struct PL_local_data
   pl_internaldebugstatus_t internal_debug; /* status of C-level debug flags */
 #endif
 
-#ifdef O_PLMT
+#ifdef O_ENGINES
   struct
-  { intptr_t   magic;			/* PL_THREAD_MAGIC (checking) */
-    struct _PL_thread_info_t *info;	/* info structure */
+  { struct _PL_thread_info_t *info;	/* info structure */
+    uint64_t seq_id;			/* Sequence id (Unique) */
 					/* Communication */
     message_queue messages;		/* Message queue */
     struct _thread_sig   *sig_head;	/* Head of signal queue */
     struct _thread_sig   *sig_tail;	/* Tail of signal queue */
     DefinitionChain local_definitions;	/* P_THREAD_LOCAL predicates */
+    int magic;				/* PL_THREAD_MAGIC (checking) */
+    int exit_requested;			/* Thread is asked to exit */
+#ifdef O_PLMT
     simpleMutex scan_lock;		/* Hold for asynchronous scans */
     thread_wait_for *waiting_for;	/* thread_wait/2 info */
     alert_channel alert;		/* How to alert the thread */
+    struct _PL_thread_info_t *creator;	/* Thread that created me */
+    uint64_t creator_seq_id;		/* Seq id of creater */
+    double child_cputime;		/* Time of completed children */
+    uint64_t child_inferences;		/* Inferences in children */
+#endif
   } thread;
 #endif
 
@@ -862,7 +876,7 @@ struct PL_local_data
 GLOBAL PL_global_data_t PL_global_data;
 GLOBAL PL_code_data_t	PL_code_data;
 GLOBAL PL_local_data_t  PL_local_data;
-#ifdef O_MULTIPLE_ENGINES
+#if defined(O_ENGINES) && !defined(O_PLMT)
 GLOBAL PL_local_data_t *PL_current_engine_ptr;
 #endif
 
